/*
 * $Id: search_cb2.c,v 1.45 2004/06/14 16:11:10 cbiere Exp $
 *
 * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
 *
 * GUI filtering functions.
 *
 *----------------------------------------------------------------------
 * This file is part of gtk-gnutella.
 *
 *  gtk-gnutella is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  gtk-gnutella is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with gtk-gnutella; if not, write to the Free Software
 *  Foundation, Inc.:
 *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *----------------------------------------------------------------------
 */

#include "gui.h"

#ifdef USE_GTK2

#include <gdk/gdkkeysyms.h>

#include "gtkcolumnchooser.h"
#include "search_cb2.h"
#include "search_gui.h"
#include "statusbar_gui.h"
#include "override.h"		/* Must be the last header included */

RCSID("$Id: search_cb2.c,v 1.45 2004/06/14 16:11:10 cbiere Exp $");

/* Privat variables */

static search_t *search_selected = NULL;

/***
 *** Private functions
 ***/

static void refresh_popup(void)
{
	static const char * const popup_names[] = {
		"popup_search_download",
		"popup_search_drop_name",
		"popup_search_drop_sha1",
		"popup_search_drop_host",
		"popup_search_drop_name_global",
		"popup_search_drop_host_global",
		"popup_search_autodownload_name",
		"popup_search_autodownload_sha1",
		"popup_search_new_from_selected",
		"popup_search_restart",
		"popup_search_duplicate",
	};
	search_t *search = search_gui_get_current_search();
	gboolean sensitive = NULL != search;
	guint i;

	for (i = 0; i < G_N_ELEMENTS(popup_names); i++)
		gtk_widget_set_sensitive(lookup_widget(popup_search, popup_names[i]),
			sensitive);

    if (search) {
        gtk_widget_set_sensitive(
            lookup_widget(popup_search, "popup_search_stop"), 
			!search_is_frozen(search->search_handle));
		gtk_widget_set_sensitive(
			lookup_widget(popup_search, "popup_search_resume"),
			search_is_frozen(search->search_handle));
		if (search->passive)
			gtk_widget_set_sensitive(
				lookup_widget(popup_search, "popup_search_restart"), FALSE);
    } else {
		gtk_widget_set_sensitive(
			lookup_widget(popup_search, "popup_search_stop"), FALSE);
		gtk_widget_set_sensitive(
			lookup_widget(popup_search, "popup_search_resume"), FALSE);
    }

}

/***
 *** Glade callbacks
 ***/
void on_combo_entry_searches_activate
    (GtkEditable *editable, gpointer user_data)
{
    /* FIXME */
}


void on_search_popdown_switch(GtkWidget *w, gpointer data)
{
	g_return_if_fail(NULL != search_selected);
	search_gui_set_current_search(search_selected);
}

void on_search_notebook_switch(GtkNotebook * notebook,
							   GtkNotebookPage * page, gint page_num,
							   gpointer user_data)
{
	search_t *sch = (search_t *) gtk_object_get_user_data(
		GTK_OBJECT(gtk_notebook_get_nth_page(notebook, page_num)));

	g_return_if_fail(sch);

	search_gui_set_current_search(sch);
}

void on_search_notebook_focus_tab(GtkNotebook * notebook,
							   GtkNotebookTab arg1,
							   gpointer user_data)
{
	search_t *sch;
	GtkWidget *widget;
	gint page_num;

	page_num = gtk_notebook_get_current_page(notebook);
	widget = gtk_notebook_get_nth_page(notebook, page_num); 
	sch = (search_t *) gtk_object_get_user_data(GTK_OBJECT(widget));

	g_return_if_fail(sch);

	search_gui_set_current_search(sch);
}

void on_tree_view_search_select_row(GtkTreeView * treeview, gpointer user_data)
{
	GtkTreeSelection *selection;
	GtkTreeModel *model;
	GtkTreeIter iter;
	search_t *sch;

	g_assert(treeview != NULL);
	selection = gtk_tree_view_get_selection(treeview);
	if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
		gtk_tree_model_get(model, &iter, c_sl_sch, &sch, -1);

		g_return_if_fail(NULL != sch);

		gtk_notebook_set_page(
			GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main")),
			nb_main_page_search);
		search_gui_set_current_search(sch);
	}
}

void on_search_selected(GtkItem * i, gpointer data)
{
	search_selected = (search_t *) data;
}

void on_button_search_clicked(GtkButton *button, gpointer user_data)
{
	gchar *e = STRTRACK(gtk_editable_get_chars(
		GTK_EDITABLE(lookup_widget(main_window, "entry_search")), 0, -1));

        /*
         * Even though we might not be on_the_net() yet, record the search.
         * There is a callback mechanism when a new node is connected, which
         * will launch the search there if it has not been sent already.
         *              --patch from Mark Schreiber, 10/01/2002
         */

    g_strstrip(e);
    if ('\0' != *e) {
        filter_t *default_filter;
        search_t *search;
        gboolean res;

		/*
		 * It's important gui_search_history_add is called before
		 * new_search, otherwise the search entry will not be
		 * cleared.
		 *      --BLUE, 04/05/2002
		 */
		gui_search_history_add(e);


		/*
		 * We have to capture the selection here already, because
		 * new_search will trigger a rebuild of the menu as a
		 * side effect.
		 */
		default_filter = (filter_t *)option_menu_get_selected_data
				(lookup_widget(main_window, "optionmenu_search_filter"));

		res = search_gui_new_search(e, 0, &search);

		/*
		 * If we should set a default filter, we do that.
		 */
		if (res && (default_filter != NULL)) {
			rule_t *rule = filter_new_jump_rule(
				default_filter, RULE_FLAG_ACTIVE);

			/*
			 * Since we don't want to distrub the shadows and
			 * do a "force commit" without the user having pressed
			 * the "ok" button in the dialog, we add the rule
			 * manually.
			 */
			search->filter->ruleset = 
                g_list_append(search->filter->ruleset, rule);
			rule->target->refcount ++;
		}
         
		if (!res)
			gdk_beep();
	}

	G_FREE_NULL(e);
}

void on_entry_search_activate(GtkEditable * editable, gpointer user_data)
{
    /*
     * Delegate to: on_button_search_clicked.
     *      --BLUE, 30/04/2002
     */

	on_button_search_clicked(NULL, user_data);
}

void on_entry_search_changed(GtkEditable * editable, gpointer user_data)
{
	gchar *e = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
	g_strstrip(e);
	gtk_widget_set_sensitive(
		lookup_widget(main_window, "button_search"), '\0' != *e);
	G_FREE_NULL(e);
}

void on_button_search_clear_clicked(GtkButton * button, gpointer user_data)
{
	gui_search_clear_results();
	gtk_widget_set_sensitive(
		lookup_widget(main_window, "button_search_clear"), FALSE);
}

void on_button_search_close_clicked(GtkButton * button, gpointer user_data)
{
    search_t *search;

    search = search_gui_get_current_search();
	g_return_if_fail(NULL != search);
	search_gui_close_search(search);
}


gboolean on_tree_view_search_results_key_press_event
    (GtkWidget *widget, GdkEventKey * event, gpointer user_data)
{
    g_assert(event != NULL);

    switch (event->keyval) {
    case GDK_Return:
        search_gui_download_files();
        return TRUE;
    default:
        return FALSE;
    };
}

gboolean on_tree_view_search_results_button_press_event
    (GtkWidget *widget, GdkEventButton * event, gpointer user_data)
{
	static guint click_time = 0;
	gboolean search_results_show_tabs;

	switch (event->button) {
	case 1:
        /* left click section */
		if (event->type == GDK_2BUTTON_PRESS) {
			gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
				"button_press_event");
			return FALSE;
		}
		if (event->type == GDK_BUTTON_PRESS) {
			if ((event->time - click_time) <= 250) {
				/*
				 * 2 clicks within 250 msec == doubleclick.
				 * Surpress further events
				 */
				gtk_signal_emit_stop_by_name(GTK_OBJECT(widget),
					"button_press_event");
				search_gui_download_files();

				return TRUE;
			} else {
				click_time = event->time;
				return FALSE;
			}
		}
		return FALSE;

	case 3:
        /* right click section (popup menu) */
        refresh_popup();
		gui_prop_get_boolean_val(PROP_SEARCH_RESULTS_SHOW_TABS,
                &search_results_show_tabs);
        gtk_label_set(GTK_LABEL((GTK_MENU_ITEM(
			lookup_widget(popup_search, "popup_search_toggle_tabs"))
                ->item.bin.child)),
			search_results_show_tabs ? _("Show search list") : _("Show tabs"));
		gtk_menu_popup(GTK_MENU(popup_search), NULL, NULL, NULL, NULL,
			event->button, event->time);
		return TRUE;

        default: ;
    }

	return FALSE;
}

void on_button_search_filter_clicked(
    GtkButton *button, gpointer user_data)
{
	filter_open_dialog();
}

void on_button_search_download_selected_clicked(
	GtkButton *button, gpointer user_data)
{
    search_gui_download_files();
}

gboolean on_tree_view_search_results_click_column(
    GtkTreeViewColumn *column, gpointer user_data)
{
	/* FIXME:
	 * 			+--> sort descending -> sort ascending -> unsorted -+
     *      	|                                                   |
     *      	+-----------------------<---------------------------+
     */
	return FALSE;
}

/* Property caches filled in on_tree_view_search_results_select_row() */
static gboolean as_similar;
static gboolean as_samesize;
static gboolean as_fuzzy;
static guint32  as_fuzzy_threshold;

/* Some static globals to collect the files and sources of all
 * autoselect_files_helper() runs and display them at the end in 
 * on_tree_view_search_results_select_row() */
/* FIXME: see below */
static guint32  sel_files;
static guint32  sel_sources;

static void autoselect_files_helper(
    GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) 
{
	record_t *rc;
	GtkTreeIter	model_iter;
	gboolean more_rows;
	GtkTreeSelection *selection;
    GtkTreeView *treeview = GTK_TREE_VIEW(data);

    /* FIXME: if we really accumulate, we'll get totally wrong numbers */
    sel_sources = 0;
    sel_files = 0;

	selection = gtk_tree_view_get_selection(treeview);
	gtk_tree_model_get(model, iter, c_sr_record, &rc, (-1));

    if (rc == NULL) {
		g_warning("autoselect_files_helper: "
			"detected row with NULL data, aborting");
		return;
	}

    /* Search the top level of the tree for items to auto-select. 
     * We don't want to search the children, because on the 
     * relevant data, all the children are the same as thier parents
     * anyway (stored in the shared_record) */
    for (
        more_rows = gtk_tree_model_get_iter_first(model, &model_iter);
		more_rows;
		more_rows = gtk_tree_model_iter_next(model, &model_iter)
    ) {
        record_t *rc2;
        GtkTreeIter	child;

		gtk_tree_model_get(model, &model_iter, c_sr_record, &rc2, (-1));

		if (rc2 == NULL) {
			g_warning("autoselect_files_helper: "
				"detected row with NULL data, skipping");
			continue;
		}

		if (
            search_gui_autoselect_cmp(rc, rc2, 
                as_similar, as_samesize,
                as_fuzzy, as_fuzzy_threshold)
        ) {
            GtkTreePath* p;

            /* Expand to the current path, otherwise any changes to
             * the selection would be discarded. */
            p = gtk_tree_model_get_path(model, &model_iter);
            gtk_tree_view_expand_row(treeview, p, FALSE);
            gtk_tree_path_free(p);

			gtk_tree_selection_select_iter(selection, &model_iter);
			sel_files ++;
            sel_sources ++;

            /* If the iter has no children, we're already done. */
            if (gtk_tree_model_iter_children(model, &child, &model_iter)) {
                /* Expand to the current path, otherwise any changes to
                 * the selection would be discarded. */
                gtk_tree_view_expand_row(treeview, path, FALSE);
    
                /* Now select all children. */
                gtk_tree_selection_select_iter(selection, &model_iter);
                do {
                    gtk_tree_selection_select_iter(selection, &child);
                    sel_sources ++;
                } while (gtk_tree_model_iter_next(model, &child));
            }	
		}
    }
}

/**
 * on_tree_view_search_results_select_row:
 *
 * This function is called when the user selectes a row in the
 * search results pane. Autoselection takes place here.
 */
void on_tree_view_search_results_select_row(
    GtkTreeView *view, gpointer user_data)
{
	static gboolean locked = FALSE;
	static gchar tmpstr[4096];
	GtkTreePath *path;

    if (locked)
        return;

    locked = TRUE;

    /*
     * Block this signal so we don't emit it for every autoselected item.
     */
    g_signal_handlers_block_by_func(
        G_OBJECT(view),
        G_CALLBACK(on_tree_view_search_results_select_row),
        NULL);

	gtk_tree_view_get_cursor(view, &path, NULL);
	if (NULL != path) {
		GtkTreeModel *model;
		const record_t *rc = NULL;
		const gchar *vendor;
		GtkTreeIter iter;
		search_t *sch = NULL;
		const GList *l = search_gui_get_searches();

		for (/* NOTHING */; NULL != l; l = g_list_next(l))
			if (view == GTK_TREE_VIEW(((search_t *) l->data)->tree_view)) {
				sch = (search_t *) l->data;
				break;
			}

		g_assert(NULL != sch);
		model = GTK_TREE_MODEL(sch->model);
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, c_sr_record, &rc, (-1));
		g_assert(NULL != rc);
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_filename")),
			lazy_locale_to_utf8(rc->name, 0));
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_sha1")),
			rc->sha1 != NULL ? sha1_base32(rc->sha1) : _("<none>"));
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_source")),
			ip_port_to_gchar(rc->results_set->ip, rc->results_set->port));
		gm_snprintf(tmpstr, sizeof(tmpstr), _("%s (%lu byte)"),
			short_size(rc->size), (gulong) rc->size);
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_size")),
			tmpstr);
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_guid")),
			guid_hex_str(rc->results_set->guid));
		g_strlcpy(tmpstr, ctime(&rc->results_set->stamp),
			MIN(25U, sizeof tmpstr)); /* discard trailing '\n' (see ctime(3) */
		gtk_entry_set_text(GTK_ENTRY(
			lookup_widget(main_window, "entry_result_info_timestamp")),
			tmpstr);
		vendor = lookup_vendor_name(rc->results_set->vendor);
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_vendor")),
			vendor != NULL ? vendor : "");
		gm_snprintf(tmpstr, sizeof(tmpstr), "%lu", (gulong) rc->index);
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_index")),
			tmpstr);
		gtk_entry_set_text(
			GTK_ENTRY(lookup_widget(main_window, "entry_result_info_tag")),
			rc->tag ? lazy_locale_to_utf8(rc->tag, 0) : "");
	}

    refresh_popup();

    /* Fill a few static global variables here, so we don't have to
     * poll the properties in autoselect_files_helper all the time. */
    gui_prop_get_boolean(
        PROP_SEARCH_AUTOSELECT_SIMILAR, 
        &as_similar, 0, 1);

    gui_prop_get_boolean(
        PROP_SEARCH_AUTOSELECT_SAMESIZE, 
        &as_samesize, 0, 1);

    gui_prop_get_boolean(
        PROP_SEARCH_AUTOSELECT_FUZZY, 
        &as_fuzzy, 0, 1);

	gnet_prop_get_guint32(
        PROP_FUZZY_THRESHOLD, 
        &as_fuzzy_threshold, 0, 1);

    /* Unfortunately Gtk2 doesn't tell us which nodes were freshly added
     * to the selection, so we have to iterate over the complete selection
     * Yikes!
     *     -- Richard, 18/04/2004
     */
    /* FIXME: there has to be a better way (see comment above) */
	gtk_tree_selection_selected_foreach(
        gtk_tree_view_get_selection(view),
        autoselect_files_helper,
		view);

    if (sel_sources > 1) {
        statusbar_gui_message(15, (as_similar
			?  _("%d files auto selected with %d sources "
			     "by urn:sha1, filename and size.")
            :  _("%d files auto selected with %d sources by urn:sha1.")),
            sel_files, sel_sources);
    }

    g_signal_handlers_unblock_by_func(
        G_OBJECT(view),
        G_CALLBACK(on_tree_view_search_results_select_row),
        NULL);

    locked = FALSE;
}

void on_button_search_passive_clicked(
    GtkButton *button, gpointer user_data)
{
    filter_t *default_filter;
	search_t *search;

    /*
     * We have to capture the selection here already, because
     * new_search will trigger a rebuild of the menu as a
     * side effect.
     */
    default_filter = (filter_t *) option_menu_get_selected_data(
		lookup_widget(main_window, "optionmenu_search_filter"));

	search_gui_new_search(_("Passive"), SEARCH_PASSIVE, &search);

    /*
     * If we should set a default filter, we do that.
     */
    if (default_filter != NULL) {
        rule_t *rule = filter_new_jump_rule(default_filter, RULE_FLAG_ACTIVE);

        /*
         * Since we don't want to distrub the shadows and
         * do a "force commit" without the user having pressed
         * the "ok" button in the dialog, we add the rule
         * manually.
         */
        search->filter->ruleset = g_list_append(search->filter->ruleset, rule);
        rule->target->refcount ++;
    }
}



/***
 *** Search results popup
 ***/

void on_popup_search_download_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_gui_download_files();
}

void on_popup_search_drop_name_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_name_eq);
    g_slist_foreach(sl, (GFunc) filter_add_drop_name_rule, search->filter);
    g_slist_free(sl);
}

void on_popup_search_drop_sha1_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_sha1_eq);
    g_slist_foreach(sl, (GFunc) filter_add_drop_sha1_rule, search->filter);
    g_slist_free(sl);
}

void on_popup_search_drop_host_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_host_eq);
    g_slist_foreach(sl, (GFunc) filter_add_drop_host_rule, search->filter);
    g_slist_free(sl);
}

void on_popup_search_drop_name_global_activate
    (GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_name_eq);
    g_slist_foreach(sl, (GFunc) filter_add_drop_name_rule,
        filter_get_global_pre());
    g_slist_free(sl);
}

void on_popup_search_drop_sha1_global_activate
    (GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_sha1_eq);
    g_slist_foreach(sl, (GFunc) filter_add_drop_sha1_rule,
        filter_get_global_pre());
    g_slist_free(sl);
}

void on_popup_search_drop_host_global_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_host_eq);
    g_slist_foreach(sl, (GFunc) filter_add_drop_host_rule,
        filter_get_global_pre());
    g_slist_free(sl);
}

void on_popup_search_autodownload_name_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_name_eq);
    g_slist_foreach(sl, (GFunc) filter_add_download_name_rule, search->filter);
    g_slist_free(sl);
}

void on_popup_search_autodownload_sha1_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

	search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_sha1_eq);
    g_slist_foreach(sl, (GFunc) filter_add_download_sha1_rule, search->filter);
    g_slist_free(sl);
}

void on_popup_search_new_from_selected_activate(
    GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;
	GtkTreeSelection *selection;
    GSList *sl;

    search = search_gui_get_current_search();
    g_assert(search != NULL);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(search->tree_view));
    sl = tree_selection_collect_data(selection, gui_record_sha1_or_name_eq);
    g_slist_foreach(sl, (GFunc) gui_add_targetted_search, search->filter);
    g_slist_free(sl);
}

void on_popup_search_edit_filter_activate(
    GtkMenuItem * menuitem,	gpointer user_data)
{
    filter_open_dialog();
}

void on_popup_search_duplicate_activate(
    GtkMenuItem * menuitem,	gpointer user_data)
{
    search_t *search;
    guint32 timeout;

    gnet_prop_get_guint32_val(PROP_SEARCH_REISSUE_TIMEOUT, &timeout);

    search = search_gui_get_current_search();
    g_return_if_fail(NULL != search);

    /* FIXME: should also duplicate filters! */
    /* FIXME: should call search_duplicate which has to be written. */
    /* FIXME: should properly duplicate passive searches. */

	search_gui_new_search_full(search->query, 0,
		timeout, search->sort_col, search->sort_order,
		search->enabled ? SEARCH_ENABLED : 0, NULL);
}

void on_popup_search_restart_activate(
	GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;

    search = search_gui_get_current_search();
	g_return_if_fail(NULL != search);
	search_gui_restart_search(search);
}

void on_popup_search_resume_activate(GtkMenuItem * menuitem,
									 gpointer user_data)
{
    search_t *search;

    search = search_gui_get_current_search();
    g_return_if_fail(NULL != search);
	gui_search_set_enabled(search, TRUE);
}

void on_popup_search_stop_activate(
	GtkMenuItem *menuitem, gpointer user_data)
{
    search_t *search;

    search = search_gui_get_current_search();
    g_return_if_fail(NULL != search);
	gui_search_set_enabled(search, FALSE);
}

void on_popup_search_config_cols_activate(GtkMenuItem * menuitem,
										  gpointer user_data)
{
	GtkWidget * cc;
    search_t *search;

    search = search_gui_get_current_search();
	g_return_if_fail(NULL != search);

	cc = gtk_column_chooser_new(GTK_WIDGET(search->tree_view));
   	gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1, 0);

	/* GtkColumnChooser takes care of cleaning up itself */
}


void on_popup_search_expand_all_activate (GtkMenuItem *menuitem, 
	gpointer user_data)
{
	search_gui_expand_all();
}

void on_popup_search_collapse_all_activate (GtkMenuItem *menuitem, 
	gpointer user_data)
{
	search_gui_collapse_all();
}


void search_callbacks_shutdown(void)
{
	/*
 	 *	Remove delayed callbacks
 	 */
}

/* vi: set ts=4: */
#endif	/* USE_GTK2 */
